# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from rest_framework import status
from rest_framework.generics import GenericAPIView
from rest_framework.response import Response
from .compat import set_cookie_with_token
from .permissions import IsSuperUser
from .authentication import JSONWebTokenAuthentication
from .serializers import \
    JSONWebTokenSerializer, RefreshAuthTokenSerializer, \
    VerifyAuthTokenSerializer, ImpersonateAuthTokenSerializer
from .settings import api_settings


class BaseJSONWebTokenAPIView(GenericAPIView):
    """Base JWT auth view used for all other JWT views (verify/refresh)."""

    permission_classes = ()
    authentication_classes = ()

    serializer_class = JSONWebTokenSerializer

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)

        serializer.is_valid(raise_exception=True)

        user = serializer.validated_data.get('user') or request.user
        token = serializer.validated_data.get('token')
        issued_at = serializer.validated_data.get('issued_at')
        response_data = JSONWebTokenAuthentication. \
            jwt_create_response_payload(token, user, request, issued_at)

        response = Response(response_data)

        if api_settings.JWT_AUTH_COOKIE:
            set_cookie_with_token(response, api_settings.JWT_AUTH_COOKIE, token)

        return response


class ObtainJSONWebTokenView(BaseJSONWebTokenAPIView):
    """
    API View that receives a POST with a user's username and password.

    Returns a JSON Web Token that can be used for authenticated requests.
    """

    serializer_class = JSONWebTokenSerializer


class VerifyJSONWebTokenView(BaseJSONWebTokenAPIView):
    """
    API View that checks the validity of a token, returning the token if it
    is valid.
    """

    serializer_class = VerifyAuthTokenSerializer


class RefreshJSONWebTokenView(BaseJSONWebTokenAPIView):
    """
    API View that returns a refreshed token (with new expiration) based on
    existing token

    If 'orig_iat' field (original issued-at-time) is found it will first check
    if it's within expiration window, then copy it to the new token.
    """

    serializer_class = RefreshAuthTokenSerializer


class ImpersonateJSONWebTokenView(GenericAPIView):
    """
    Impersonate the user by retrieving its JWT.

    By default, the view permits this action only to superusers in order to
    be backwards-compatible as much as possible. If you need to customize
    the permission handling process, override the `permission_classes` attribute
    using `djangorestframework`'s permission system.

    Returns:
        dict: {"token": user's JWT}
    """

    permission_classes = (IsSuperUser, )
    serializer_class = ImpersonateAuthTokenSerializer

    def post(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)

        serializer.is_valid(raise_exception=True)

        token = serializer.validated_data.get("token")
        response = Response({"token": token})

        if api_settings.JWT_IMPERSONATION_COOKIE:
            set_cookie_with_token(
                response,
                api_settings.JWT_IMPERSONATION_COOKIE,
                token)

        return response


obtain_jwt_token = ObtainJSONWebTokenView.as_view()
verify_jwt_token = VerifyJSONWebTokenView.as_view()
refresh_jwt_token = RefreshJSONWebTokenView.as_view()
impersonate_jwt_token = ImpersonateJSONWebTokenView.as_view()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import pytest

from django.contrib.auth import get_user_model

from rest_framework.test import APIClient

from rest_framework_jwt.authentication import JSONWebTokenAuthentication
from rest_framework_jwt.settings import api_settings


User = get_user_model()


@pytest.fixture
def create_authenticated_client(api_client):
    def _create_authenticated_client(user):
        payload = JSONWebTokenAuthentication.jwt_create_payload(user)
        token = JSONWebTokenAuthentication.jwt_encode_payload(payload)
        api_client.credentials(
            HTTP_AUTHORIZATION="{prefix} {token}".format(
                prefix=api_settings.JWT_AUTH_HEADER_PREFIX, token=token
            )
        )
        return api_client

    yield _create_authenticated_client

    # Clear the authorization header
    api_client.credentials()


@pytest.fixture
def api_client():
    return APIClient()


@pytest.fixture
def user(create_user):
    return create_user(
        username="username",
        email="username@example.com",
        password="password",
        is_active=True,
        is_staff=False,
        is_superuser=False,
    )


@pytest.fixture
def staff_user(create_user):
    return create_user(
        username="staffusername",
        email="staffusername@example.com",
        password="staff",
        is_active=True,
        is_staff=True,
        is_superuser=False,
    )


@pytest.fixture
def super_user(create_user):
    return create_user(
        username="superusername",
        email="superusername@example.com",
        password="super",
        is_active=True,
        is_staff=True,
        is_superuser=True,
    )


@pytest.fixture
def create_user(db):
    def _create_user(**kwargs):
        return User.objects.create_user(**kwargs)

    return _create_user



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from rest_framework import status
from rest_framework.reverse import reverse

from rest_framework_jwt.authentication import JSONWebTokenAuthentication
from rest_framework_jwt.settings import api_settings


def test_superuser_can_impersonate(
        user, super_user, create_authenticated_client
):

    api_client = create_authenticated_client(super_user)

    data = {"user": user.id}
    url = reverse("impersonate")
    response = api_client.post(url, data, format="json")

    token = response.json()["token"]
    payload = JSONWebTokenAuthentication.jwt_decode_token(token)

    assert response.status_code == status.HTTP_200_OK
    assert "token" in response.json()
    assert payload["user_id"] == user.id


def test_staff_user_cannot_impersonate(
        user, staff_user, create_authenticated_client
):

    api_client = create_authenticated_client(staff_user)

    data = {"user": user.id}
    url = reverse("impersonate")
    response = api_client.post(url, data, format="json")

    assert response.status_code == status.HTTP_403_FORBIDDEN


def test_normal_user_cannot_impersonate(user, create_authenticated_client):

    api_client = create_authenticated_client(user)

    data = {"user": user.id}
    url = reverse("impersonate")
    response = api_client.post(url, data, format="json")

    assert response.status_code == status.HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_impersonate(api_client):

    url = reverse("impersonate")
    response = api_client.post(url, format="json")

    assert response.status_code == status.HTTP_401_UNAUTHORIZED


def test_superuser_cannot_impersonate_inactive_user(
    user, super_user, create_authenticated_client
):

    api_client = create_authenticated_client(super_user)

    user.is_active = False
    user.save()

    data = {"user": user.id}
    url = reverse("impersonate")
    response = api_client.post(url, data, format="json")

    expected_output = {"non_field_errors": [_("User account is disabled.")]}

    assert response.status_code == status.HTTP_400_BAD_REQUEST
    assert response.json() == expected_output


def test_impersonation_sets_cookie(
    monkeypatch, user, super_user, create_authenticated_client
):

    imp_cookie = "jwt-imp"
    monkeypatch.setattr(api_settings, "JWT_IMPERSONATION_COOKIE", imp_cookie)

    api_client = create_authenticated_client(super_user)

    data = {"user": user.id}
    url = reverse("impersonate")
    response = api_client.post(url, data, format="json")

    token = response.json()["token"]
    payload = JSONWebTokenAuthentication.jwt_decode_token(token)

    cookie_token = response.client.cookies.get(imp_cookie)
    cookie_payload = JSONWebTokenAuthentication.jwt_decode_token(cookie_token.value)

    assert response.status_code == status.HTTP_200_OK
    assert "token" in response.json()
    assert imp_cookie in response.client.cookies
    assert payload["user_id"] == user.id
    assert cookie_payload["user_id"] == user.id


def test_view_with_impersonation_cookie(
    monkeypatch, user, super_user, call_auth_endpoint
):

    auth_cookie = "jwt-auth"
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE", auth_cookie)

    imp_cookie = "jwt-imp"
    monkeypatch.setattr(api_settings, "JWT_IMPERSONATION_COOKIE", imp_cookie)

    api_client = call_auth_endpoint("superusername", "super")

    data = {"user": user.id}
    url = reverse("impersonate")
    response = api_client.client.post(url, data, format="json")

    url = reverse("superuser-test-view")
    response = response.client.get(url)

    assert response.status_code == status.HTTP_403_FORBIDDEN

    url = reverse("test-view")
    response = response.client.get(url)

    assert response.status_code == status.HTTP_200_OK



from rest_framework.permissions import BasePermission


class IsSuperUser(BasePermission):
    """
    Permission check for superusers.
    """
    def has_permission(self, request, view):
        return bool(request.user and request.user.is_superuser)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from datetime import datetime

from django import VERSION

from .settings import api_settings

try:
    from django.urls import include, url
except ImportError:
    from django.conf.urls import include, url  # noqa: F401

def has_set_cookie_samesite():
    return (VERSION >= (2,1,0))

def set_cookie_with_token(response, name, token):
    params = {
        'expires': datetime.utcnow() + api_settings.JWT_EXPIRATION_DELTA,
        'domain': api_settings.JWT_AUTH_COOKIE_DOMAIN,
        'path': api_settings.JWT_AUTH_COOKIE_PATH,
        'secure': api_settings.JWT_AUTH_COOKIE_SECURE,
        'httponly': True
    }

    if has_set_cookie_samesite():
        params.update({'samesite': api_settings.JWT_AUTH_COOKIE_SAMESITE})

    response.set_cookie(name, token, **params)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from six.moves import reload_module

from django.core.exceptions import ImproperlyConfigured
from django.test import TestCase, override_settings

from rest_framework_jwt import settings


def invalid_JWT_EXPIRATION_DELTA_setting():
    return {"JWT_EXPIRATION_DELTA": "invalid"}


def invalid_JWT_REFRESH_EXPIRATION_DELTA_setting():
    return {"JWT_REFRESH_EXPIRATION_DELTA": "invalid"}


class SettingsTestCase(TestCase):
    @classmethod
    def tearDownClass(cls):
        # prevent overridden settings from persisting across tests
        reload_module(settings)

    @override_settings(JWT_AUTH=invalid_JWT_EXPIRATION_DELTA_setting())
    def test_invalid_JWT_EXPIRATION_DELTA_setting(self):
        with self.assertRaises(ImproperlyConfigured):
            reload_module(settings)

    @override_settings(JWT_AUTH=invalid_JWT_REFRESH_EXPIRATION_DELTA_setting())
    def test_invalid_JWT_REFRESH_EXPIRATION_DELTA_setting(self):
        with self.assertRaises(ImproperlyConfigured):
            reload_module(settings)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from rest_framework_jwt.authentication import JSONWebTokenAuthentication
from rest_framework_jwt.settings import api_settings


def test_invalid_token__returns_validation_error(call_auth_refresh_endpoint):
    expected_output = {"non_field_errors": [_("Error decoding token.")]}

    response = call_auth_refresh_endpoint("invalid_token")
    assert response.json() == expected_output


def test_with_JWT_ALLOW_REFRESH_disabled__returns_validation_error(
    monkeypatch, call_auth_refresh_endpoint, user
):
    monkeypatch.setattr(api_settings, "JWT_ALLOW_REFRESH", False)

    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    payload["exp"] = payload["iat"] + 100  # add 100 seconds to issued at time
    auth_token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    expected_output = {
        "non_field_errors": ["orig_iat field not found in token."]
    }

    refresh_response = call_auth_refresh_endpoint(auth_token)

    assert refresh_response.json() == expected_output


def test_without_orig_iat_in_payload__returns_validation_error(
    call_auth_refresh_endpoint, user
):
    # create token without orig_iat in payload
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    del payload["orig_iat"]
    auth_token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    expected_output = {
        "non_field_errors": [_("orig_iat field not found in token.")]
    }

    response = call_auth_refresh_endpoint(auth_token)
    assert response.json() == expected_output


def test_refresh_limit_expired__returns_validation_error(
    call_auth_refresh_endpoint, user
):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    payload["orig_iat"] = 0  # beginning of time
    auth_token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    expected_output = {"non_field_errors": [_("Refresh has expired.")]}

    response = call_auth_refresh_endpoint(auth_token)
    assert response.json() == expected_output


def test_valid_token__returns_new_token(call_auth_refresh_endpoint, user):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    payload["exp"] = payload["iat"] + 100  # add 100 seconds to issued at time
    auth_token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    refresh_response = call_auth_refresh_endpoint(auth_token)
    refresh_token = refresh_response.json()["token"]
    assert refresh_token != auth_token


def test_expired_token__returns_validation_error(
    call_auth_refresh_endpoint, user
):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    payload["iat"] = 0
    payload["exp"] = 1
    auth_token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    expected_output = {"non_field_errors": [_("Token has expired.")]}

    refresh_response = call_auth_refresh_endpoint(auth_token)
    assert refresh_response.json() == expected_output



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from rest_framework_jwt.authentication import JSONWebTokenAuthentication


def test_invalid_token_returns_validation_error(call_auth_verify_endpoint):
    expected_output = {"non_field_errors": [_("Error decoding token.")]}

    response = call_auth_verify_endpoint("invalid_token")

    assert response.json() == expected_output


def test_valid_token_returns_same_token(
    user, call_auth_endpoint, call_auth_verify_endpoint
):
    auth_response = call_auth_endpoint("username", "password")
    auth_token = auth_response.json()["token"]

    verify_response = call_auth_verify_endpoint(auth_token)
    verify_token = verify_response.json()["token"]

    assert verify_token == auth_token


def test_token_without_username_returns_validation_error(
    user, call_auth_verify_endpoint
):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    payload.pop("username")
    auth_token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    expected_output = {"non_field_errors": [_("Invalid token.")]}

    verify_response = call_auth_verify_endpoint(auth_token)

    assert verify_response.json() == expected_output


def test_token_with_invalid_username_returns_validation_error(
    user, call_auth_verify_endpoint
):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    payload["username"] = "i_do_not_exist"
    auth_token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    expected_output = {"non_field_errors": [_("User doesn't exist.")]}

    verify_response = call_auth_verify_endpoint(auth_token)
    assert verify_response.json() == expected_output


def test_token_for_inactive_user_returns_validation_error(
    create_user, call_auth_verify_endpoint
):
    inactive_user = create_user(
        username="inactive", password="password", is_active=False
    )
    payload = JSONWebTokenAuthentication.jwt_create_payload(inactive_user)
    auth_token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    expected_output = {"non_field_errors": [_("User account is disabled.")]}

    verify_response = call_auth_verify_endpoint(auth_token)
    assert verify_response.json() == expected_output


def test_expired_token_returns_validation_error(
    user, call_auth_verify_endpoint
):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    payload["iat"] = 0  # beginning of time
    payload["exp"] = 1  # one second after beginning of time
    auth_token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    expected_output = {"non_field_errors": [_("Token has expired.")]}

    verify_response = call_auth_verify_endpoint(auth_token)
    assert verify_response.json() == expected_output



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import pytest

from rest_framework.reverse import reverse


@pytest.fixture
def call_auth_endpoint(api_client, db):
    def _call_auth_endpoint(username, password):
        url = reverse("auth")
        data = {"username": username, "password": password}
        return api_client.post(path=url, data=data)

    return _call_auth_endpoint


@pytest.fixture
def call_auth_verify_endpoint(api_client, db):
    def _call_auth_verify_endpoint(token):
        url = reverse("auth-verify")
        data = {"token": token}
        return api_client.post(path=url, data=data)

    return _call_auth_verify_endpoint


@pytest.fixture
def call_auth_refresh_endpoint(api_client, db):
    def _call_auth_refresh_endpoint(token):
        url = reverse("auth-refresh")
        data = {"token": token}
        return api_client.post(path=url, data=data)

    return _call_auth_refresh_endpoint



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from rest_framework import status

from rest_framework_jwt.authentication import JSONWebTokenAuthentication
from rest_framework_jwt.compat import has_set_cookie_samesite
from rest_framework_jwt.settings import api_settings

def test_empty_credentials_returns_validation_error(call_auth_endpoint):
    expected_output = {
        "password": [_("This field may not be blank.")],
        "username": [_("This field may not be blank.")],
    }

    response = call_auth_endpoint("", "")

    assert response.json() == expected_output


def test_auth__invalid_credentials__returns_validation_error(
    call_auth_endpoint,
):
    expected_output = {
        "non_field_errors": [_("Unable to log in with provided credentials.")]
    }

    response = call_auth_endpoint("invalid_username", "invalid_password")

    assert response.json() == expected_output


def test_valid_credentials_return_jwt(user, call_auth_endpoint):
    response = call_auth_endpoint("username", "password")

    token = response.json()["token"]
    payload = JSONWebTokenAuthentication.jwt_decode_token(token)

    assert response.status_code == status.HTTP_200_OK
    assert payload["user_id"] == user.id
    assert payload["username"] == user.get_username()


def test_valid_credentials_with_aud_and_iss_settings_return_jwt(
    monkeypatch, user, call_auth_endpoint
):
    monkeypatch.setattr(api_settings, "JWT_AUDIENCE", "test-aud")
    monkeypatch.setattr(api_settings, "JWT_ISSUER", "test-iss")

    response = call_auth_endpoint("username", "password")

    token = response.json()["token"]
    payload = JSONWebTokenAuthentication.jwt_decode_token(token)

    assert response.status_code == status.HTTP_200_OK
    assert payload["aud"] == "test-aud"
    assert payload["iss"] == "test-iss"
    assert payload["user_id"] == user.id
    assert payload["username"] == user.get_username()


def test_valid_credentials_with_JWT_GET_USER_SECRET_KEY_set_return_jwt(
    monkeypatch, user, call_auth_endpoint
):
    def jwt_get_user_secret_key(user):
        return "{0}-{1}-{2}".format(user.pk, user.get_username(), "key")

    monkeypatch.setattr(
        api_settings, "JWT_GET_USER_SECRET_KEY", jwt_get_user_secret_key
    )

    response = call_auth_endpoint("username", "password")

    token = response.json()["token"]
    payload = JSONWebTokenAuthentication.jwt_decode_token(token)

    assert response.status_code == status.HTTP_200_OK
    assert payload["user_id"] == user.id
    assert payload["username"] == user.get_username()


def test_valid_credentials_with_no_user_id_setting_returns_jwt(
    monkeypatch, user, call_auth_endpoint
):
    monkeypatch.setattr(api_settings, "JWT_PAYLOAD_INCLUDE_USER_ID", False)

    response = call_auth_endpoint("username", "password")

    token = response.json()["token"]
    payload = JSONWebTokenAuthentication.jwt_decode_token(token)

    assert response.status_code == status.HTTP_200_OK
    assert "user_id" not in payload
    assert payload["username"] == user.get_username()


def test_valid_credentials_inactive_user_return_validation_error(
    create_user, call_auth_endpoint
):
    expected_output = {
        "non_field_errors": [_("Unable to log in with provided credentials.")]
    }

    create_user(username="inactive", password="password", is_active=False)

    response = call_auth_endpoint("inactive", "password")

    assert response.json() == expected_output


def test_valid_credentials_with_auth_cookie_enabled_returns_jwt_and_cookie(
    monkeypatch, user, call_auth_endpoint
):

    auth_cookie = "jwt-auth"
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE", auth_cookie)

    response = call_auth_endpoint("username", "password")

    assert auth_cookie in response.cookies

    setcookie = response.cookies[auth_cookie]

    assert 'domain' not in setcookie.items()
    assert setcookie['path'] == '/'
    assert setcookie['secure'] is True
    assert setcookie['httponly'] is True	# hardcoded
    if has_set_cookie_samesite():
        assert setcookie['samesite'] == 'Lax'

    assert response.status_code == status.HTTP_200_OK
    assert "token" in force_text(response.content)
    assert auth_cookie in response.client.cookies

def test_auth_cookie_settings(
    monkeypatch, user, call_auth_endpoint
):

    auth_cookie = "jwt-auth"
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE", auth_cookie)
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE_DOMAIN", '.do.main')
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE_PATH", '/pa/th')
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE_SECURE", False)
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE_SAMESITE", 'Strict')

    response = call_auth_endpoint("username", "password")

    assert auth_cookie in response.cookies

    setcookie = response.cookies[auth_cookie]

    assert setcookie['domain'] == '.do.main'
    assert setcookie['path'] == '/pa/th'
    assert 'secure' not in setcookie.items()
    assert setcookie['httponly'] is True	# hardcoded
    if has_set_cookie_samesite():
        assert setcookie['samesite'] == 'Strict'



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from rest_framework import status
from rest_framework.reverse import reverse

from rest_framework_jwt.authentication import JSONWebTokenAuthentication
from rest_framework_jwt.settings import api_settings


def test_view_requires_authentication(api_client):
    url = reverse("test-view")
    response = api_client.get(url)

    expected_output = {
        "detail": _("Authentication credentials were not provided.")
    }

    assert response.status_code == status.HTTP_401_UNAUTHORIZED
    assert response.json() == expected_output


def test_view_returns_200_to_authenticated_user(
    create_authenticated_client, user
):
    api_client = create_authenticated_client(user)

    url = reverse("test-view")
    response = api_client.get(url)

    assert response.status_code == status.HTTP_200_OK


def test_view_returns_401_for_invalid_token(api_client):
    token = "invalid"
    api_client.credentials(HTTP_AUTHORIZATION="Bearer " + token)

    expected_output = {"detail": _("Error decoding token.")}

    url = reverse("test-view")
    response = api_client.get(url)

    assert response.status_code == status.HTTP_401_UNAUTHORIZED
    assert response.json() == expected_output


def test_view_returns_401_for_expired_token(api_client, user):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    payload["iat"] = 0
    payload["exp"] = 1
    token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    api_client.credentials(HTTP_AUTHORIZATION="Bearer " + token)

    expected_output = {"detail": _("Token has expired.")}

    url = reverse("test-view")
    response = api_client.get(url)

    assert response.status_code == status.HTTP_401_UNAUTHORIZED
    assert response.json() == expected_output


def test_view_returns_401_to_deactivated_user(api_client, user):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    token = JSONWebTokenAuthentication.jwt_encode_payload(payload)
    user.is_active = False
    user.save()

    api_client.credentials(HTTP_AUTHORIZATION="Bearer " + token)

    expected_output = {"detail": _("User account is disabled.")}

    url = reverse("test-view")
    response = api_client.get(url)

    assert response.status_code == status.HTTP_401_UNAUTHORIZED
    assert response.json() == expected_output


def test_view_returns_401_when_username_does_not_exist(api_client, user):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    payload["username"] = "i_do_not_exist"
    token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    api_client.credentials(HTTP_AUTHORIZATION="Bearer " + token)

    expected_output = {"detail": _("Invalid token.")}

    url = reverse("test-view")
    response = api_client.get(url)

    assert response.status_code == status.HTTP_401_UNAUTHORIZED
    assert response.json() == expected_output


def test_view_returns_401_when_token_does_not_contain_username(
    api_client, user
):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    payload.pop("username")
    token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    api_client.credentials(HTTP_AUTHORIZATION="Bearer " + token)

    expected_output = {"detail": _("Invalid payload.")}

    url = reverse("test-view")
    response = api_client.get(url)

    assert response.status_code == status.HTTP_401_UNAUTHORIZED
    assert response.json() == expected_output


def test_view_returns_401_to_authorization_header_without_token(api_client):
    api_client.credentials(HTTP_AUTHORIZATION="Bearer ")

    expected_output = {
        "detail": _("Invalid Authorization header. No credentials provided.")
    }

    url = reverse("test-view")
    response = api_client.get(url)

    assert response.status_code == status.HTTP_401_UNAUTHORIZED
    assert response.json() == expected_output


def test_view_returns_401_to_authorization_header_token_with_spaces(
    api_client, user
):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    token = JSONWebTokenAuthentication.jwt_encode_payload(payload)
    token = token.replace(".", " ")

    api_client.credentials(HTTP_AUTHORIZATION="Bearer " + token)

    expected_output = {
        "detail": _(
            "Invalid Authorization header. Credentials string should "
            "not contain spaces."
        )
    }

    url = reverse("test-view")
    response = api_client.get(url)

    assert response.status_code == status.HTTP_401_UNAUTHORIZED
    assert response.json() == expected_output


def test_view_returns_401_to_invalid_token_prefix(api_client, user):
    payload = JSONWebTokenAuthentication.jwt_create_payload(user)
    token = JSONWebTokenAuthentication.jwt_encode_payload(payload)

    api_client.credentials(HTTP_AUTHORIZATION="INVALID_PREFIX " + token)

    expected_output = {
        "detail": _("Authentication credentials were not provided.")
    }

    url = reverse("test-view")
    response = api_client.get(url)

    assert response.status_code == status.HTTP_401_UNAUTHORIZED
    assert response.json() == expected_output


def test_view__auth_cookie(monkeypatch, user, call_auth_endpoint):
    auth_cookie = "jwt-auth"
    monkeypatch.setattr(api_settings, "JWT_AUTH_COOKIE", auth_cookie)

    response = call_auth_endpoint("username", "password")

    url = reverse("test-view")
    response = response.client.get(url)

    assert response.status_code == status.HTTP_200_OK
